PROGRAM_NAME='CTEIB8_IP_Basics'
(***********************************************************)
(* System Type: NetLinx                                    *)
(***********************************************************)
(* REV HISTORY:                                            *)
(***********************************************************)

(***********************************************************)
(* IMPORTANT !                                             *)
(*                                                         *)
(*  The following code blocks are provided as a guide to   *)
(*  programming the device(s). This is a                   *)
(*  sample only, and will most likely require modification *)
(*  to be integrated into a master program.                *)
(*                                                         *)
(*  Device-specific protocols should be obtained directly  *)
(*  from the equipment manufacturer to ensure compliance   *)
(*  with the most current versions. Within the limits      *)
(*  imposed by any non-disclosure agreements, AMX will     *)
(*  provide protocols upon request.                        *)
(*                                                         *)
(*  If further programming assistance is required, please  *)
(*  contact your AMX customer support team.                *)
(***********************************************************)

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE
dvEIB			= 0:5:0
dvTP			= 10001:1:0
vdvEIB		= 33001:1:0		// virtual device for module control and response


(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT
/*
For better readabillity of the code, it is recommended to create a constant for the index
of the actuators.
If you use the constant instead of the number, it helps a lot reading the code.
Also if the index change, you have to change it only in one place
*/
knxLicht_1_Switch_Set			= 1;
knxLicht_1_Switch_Status	= 2;
knxLicht_1_Dim4						= 3;
knxLicht_1_1Byte_Set			= 4;
knxLicht_1_1Byte_Status		= 5;
knxLicht_2_Switch_Set			= 6;
knxLicht_2_Switch_Status	= 7;
knxLicht_2_Dim4						= 8;
knxLicht_2_1Byte_Set			= 9;
knxLicht_2_1Byte_Status		= 10;
knxJalousie_1_UpDown			= 11;
knxJalousie_1_Stop				= 12;
knxJalousie_2_UpDown			= 13;
knxJalousie_2_Stop				= 14;
knxTemp_1_Status					= 15;
knxTemp_2_Status					= 16;
knxTime_1									= 17;
knxDate_1									= 18;
knxText_1									= 19;
knxHexText_1							= 20;
knx4Byte_1								= 21;

#include 'EIB_Tools.axi';		// This file contains usefull functions for EIB

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE
VOLATILE LONG lEIB_Value[3000];	// max. 3000 group addresses

(***********************************************************)
(*                 STARTUP CODE GOES BELOW                 *)
(***********************************************************)
DEFINE_START


DEFINE_MODULE 'CTEIB8_mod' EIB8(vdvEIB, dvEIB, lEIB_Value)

(***********************************************************)
(*                  THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT
DATA_EVENT[vdvEIB]
{
	ONLINE :
	{
		// tells the module the IP address of the gateway
		SEND_COMMAND vdvEIB,'IP=10.6.2.230' // IP Address of CTG-EIB-IP gateway
		SEND_COMMAND vdvEIB,"'STARTDELAY=30'"	// Default 30 Sekunden until table upload
	}
	string:
	{
		stack_var char sCmd[50];
		stack_var integer nIdx;
		
		// PArsind string feedback from the module
		// some values could not be reported as a channel or level
		// so it comes as a string like
		// SET=2:20.3
		// VAL=2:20.3
		
		// Seperate command from parameter
		sCmd = remove_string(data.text, "'='", 1);
		
		switch(upper_string(sCmd))
		{
			// Temperature or similar encoded value
			case 'EIS5=':	
			{
				nIdx = atoi(remove_string(data.text, "':'", 1));
				switch(nIdx)
				{
					case knxTemp_1_Status:
					{ send_command dvTP,"'^TXT-101,0,', data.text, 'C'"; }
					case knxTemp_2_Status:
					{ send_command dvTP,"'^TXT-102,0,', data.text, 'C'"; }
				}
			}
			// A time
			case 'TIME=':
			{
				nIdx = atoi(remove_string(data.text, "':'", 1));
				switch(nIdx)
				{
					case knxTime_1:
					{ send_command dvTP,"'^TXT-103,0,', data.text, ' Uhr'"; }
				}
			}
			// A date
			case 'DATE=':
			{
				nIdx = atoi(remove_string(data.text, "':'", 1));
				switch(nIdx)
				{
					case knxDate_1:
					{ send_command dvTP,"'^TXT-104,0,', data.text, ' Tag'"; }
				}
			}
			// Any othe knx type
			case 'SET=':	// The value has been reported on EIB, and has changed
			case 'VAL=':	// The value has been reported on EIB, but not changed
			{
				nIdx = atoi(remove_string(data.text, "':'", 1));
				switch(nIdx)
				{
					case knxText_1:
					{ send_command dvTP,"'^TXT-105,0,Text:', data.text"; }
					case knxHexText_1:
					{ send_command dvTP,"'^TXT-106,0,Hex Text:', data.text"; }
				}
			}
		}
	}
}

// We are ready to load the group address table
// Note, if you have a big table, it is best to not load more than 50 in one run
// Stack them with a wait of "5"
channel_event[vdvEIB, 3001]
{
	on:
	{
		EIBAdd(vdvEIB,  knxLicht_1_Switch_Set, 		eibSwitch, 		'1/1/0', "");								// Licht,  Zustand 
		EIBAdd(vdvEIB,  knxLicht_1_Switch_Status, eibSwitch, 		'1/1/30', "eibPollstart");	// Licht,  Zustand 
		EIBAdd(vdvEIB,  knxLicht_1_Dim4, 					eibDim4,   		'1/1/10', "");   						// Dimmer  relativ 
		EIBAdd(vdvEIB,  knxLicht_1_1Byte_Set, 		eib1Byte,  		'1/1/20', "");   						// Dimmer  absolut 
		EIBAdd(vdvEIB,  knxLicht_1_1Byte_Status, 	eib1Byte,  		'1/1/40', "eibPollstart");	// Dimmer  Wert lesen 
									 
		EIBAdd(vdvEIB,  knxLicht_2_Switch_Set, 		eibSwitch, 		'1/1/1', "");								// Licht,  Zustand 
		EIBAdd(vdvEIB,  knxLicht_2_Switch_Status, eibSwitch, 		'1/1/31', "eibPollstart");	// Licht,  Zustand 
		EIBAdd(vdvEIB,  knxLicht_2_Dim4, 					eibDim4,   		'1/1/11', "");   						// Dimmer  relativ 
		EIBAdd(vdvEIB,  knxLicht_2_1Byte_Set, 		eib1Byte,  		'1/1/21', "");   						// Dimmer  absolut 
		EIBAdd(vdvEIB,  knxLicht_2_1Byte_Status, 	eib1Byte,  		'1/1/41', "eibPollstart");	// Dimmer  Wert lesen 
									 
		EIBAdd(vdvEIB, 	knxJalousie_1_UpDown, 		eibSwitch, 		'1/2/0', "");  					// Jalousie,  Auf/Ab 
		EIBAdd(vdvEIB, 	knxJalousie_1_Stop, 			eibSwitch, 		'1/2/2', "");  					// Jalousie,  Lamellen 
		EIBAdd(vdvEIB, 	knxJalousie_2_UpDown, 		eibSwitch, 		'1/2/1', "");  					// Jalousie,  Auf/Ab 
		EIBAdd(vdvEIB, 	knxJalousie_2_Stop, 			eibSwitch, 		'1/2/3', "");  					// Jalousie,  Lamellen 
									 
		EIBAdd(vdvEIB, 	knxTemp_1_Status, 				eib2Byte,  		'1/3/4', "eibEIS5, ',',eibPollstart");  // Analogwert 1 
		EIBAdd(vdvEIB, 	knxTemp_2_Status, 				eib2Byte,  		'1/3/5', "eibEIS5, ',',eibPollstart");  // Analogwert 2 
		EIBAdd(vdvEIB, 	knxTime_1, 								eib3Byte,  		'1/4/0', "eibTIME, ',',eibPollstart");  // Zeit
		EIBAdd(vdvEIB, 	knxDate_1, 								eib3Byte,  		'1/4/1', "eibDATE, ',',eibPollstart");  // Datum
									 
		EIBAdd(vdvEIB, 	knxText_1, 								eibtext,  		'1/6/1', "eibPollstart"); 
		EIBAdd(vdvEIB, 	knxHexText_1, 						eibhextext,  	'1/6/2', "eibPollstart"); 
		EIBAdd(vdvEIB, 	knx4Byte_1, 							eib4Byte,  		'1/6/0', "eibPollstart"); 
	}
}

// We get the value of each actuator also as a level
level_event[vdvEIB, knxLicht_1_1Byte_Status]
{
	send_level dvTP,1, level.value;
}

level_event[vdvEIB, knxLicht_2_1Byte_Status]
{
	send_level dvTP,2, level.value;
}

// Process button presses on the panel
// The easy way for this demo
BUTTON_EVENT[dvTP, 0]
{
	push:
	{
		min_to[button.input];
		switch(button.input.channel)
		{
			// Dimmer 1
			case  1: { EIBSet(vdvEIB,  knxLicht_1_Switch_Set, ![vdvEIB, knxLicht_1_Switch_Status]); }	// Toggle a value
			case  2: { EIBSet(vdvEIB,  knxLicht_1_Dim4, 			EIB_DIM_UP); }		// Start dimming up
			case  3: { EIBSet(vdvEIB,  knxLicht_1_Dim4, 			EIB_DIM_DN); }		// Start dimming down
			case  4: { EIBSet(vdvEIB,  knxLicht_1_1Byte_Set, 	127); }						// Set the value to "127"
			// Dimmer 2
			case 11: { EIBSet(vdvEIB,  knxLicht_2_Switch_Set, ![vdvEIB, knxLicht_2_Switch_Status]); }	// Toggle a value
			case 12: { EIBSet(vdvEIB,  knxLicht_2_Dim4, 			EIB_DIM_UP); }		// Start dimming up
			case 13: { EIBSet(vdvEIB,  knxLicht_2_Dim4, 			EIB_DIM_DN); }		// Start dimming down
			case 14: { EIBSet(vdvEIB,  knxLicht_2_1Byte_Set, 	127); }						// Set the value to "127"
			// Jalousie 1
			case 21: { EIBSet(vdvEIB,  knxJalousie_1_UpDown, 	0); }
			case 22: { EIBSet(vdvEIB,  knxJalousie_1_UpDown, 	1); }
			case 23: { EIBSet(vdvEIB,  knxJalousie_1_Stop, 		0); }
			// Jalousie 2
			case 31: { EIBSet(vdvEIB,  knxJalousie_2_UpDown, 	0); }
			case 32: { EIBSet(vdvEIB,  knxJalousie_2_UpDown, 	1); }
			case 33: { EIBSet(vdvEIB,  knxJalousie_2_Stop, 		0); }
		}
	}
	release:
	{
		switch(button.input.channel)
		{
			// Dimmer 1
			case  2: { EIBSet(vdvEIB,  knxLicht_1_Dim4, EIB_DIM_SP); }		//  Stop dimming
			case  3: { EIBSet(vdvEIB,  knxLicht_1_Dim4, EIB_DIM_SP); }		//  Stop dimming
			// Dimmer 2
			case 12: { EIBSet(vdvEIB,  knxLicht_2_Dim4, EIB_DIM_SP); }		//  Stop dimming
			case 13: { EIBSet(vdvEIB,  knxLicht_2_Dim4, EIB_DIM_SP); }		//  Stop dimming
		}
	}
}

// Let the panel beep, just we know boot has finished
data_event[dvTP]
{
	online:
	{
		send_command dvTP,'ADBEEP';
	}
}
(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM
/*
NOTE: on new masters, code in define_program should be avoided.
      To keep this demo simple, we use a "wait 1" for the feedback
			In real programms, it is best to use a timeline for that.
*/
wait 1
{
	[dvTP,1] 		= ([vdvEIB, knxLicht_1_Switch_Status]);
	[dvTP,11] 	= ([vdvEIB, knxLicht_2_Switch_Status]);
	
	[dvTP,251] 	= ([vdvEIB, 3001]);	// GW read to load table
	[dvTP,252] 	= ([vdvEIB, 3002]);	// GW is ready for use
	
}


(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
